@@ -29,18 +29,11 @@
 
 #ifndef SC_EXPORT_H
 #define SC_EXPORT_H
-#include <typeinfo>
+#include <typeindex>
 
 #include "sysc/communication/sc_communication_ids.h"
 #include "sysc/communication/sc_interface.h"
 #include "sysc/kernel/sc_object.h"
-#include "sysc/utils/sc_typeindex.h"
-
-#if ! defined( SC_DISABLE_VIRTUAL_BIND )
-#  define SC_VIRTUAL_ virtual
-#else
-#  define SC_VIRTUAL_ /* non-virtual */
-#endif
 
 namespace sc_core {
 
@@ -65,7 +58,7 @@ public:
     virtual       const sc_interface* get_interface() const = 0;
 
     // return RTTI information of associated interface
-    virtual sc_type_index get_interface_type() const = 0;
+    virtual std::type_index get_interface_type() const = 0;
 
 protected:
     
@@ -181,7 +174,7 @@ public: // interface access:
 
 
 public: // binding:
-    SC_VIRTUAL_ void bind( IF& interface_ )
+    virtual void bind( IF& interface_ )
     {
         if ( m_interface_p )
         {
@@ -200,7 +193,7 @@ public: // identification:
     virtual const char* kind() const { return "sc_export"; }
 
     // return RTTI information of associated interface
-    virtual sc_type_index get_interface_type() const
+    virtual std::type_index get_interface_type() const
     {
         return typeid( IF );
     }
@@ -268,8 +261,6 @@ private:
 
 } // namespace sc_core
 
-#undef SC_VIRTUAL_
-
 // $Log: sc_export.h,v $
 // Revision 1.7  2011/08/26 20:45:40  acg
 //  Andy Goodrich: moved the modification log to the end of the file to
