@@ -36,12 +36,6 @@
 #include "sysc/datatypes/bit/sc_logic.h"
 #include "sysc/tracing/sc_trace.h"
 
-#if ! defined( SC_DISABLE_VIRTUAL_BIND )
-#  define SC_VIRTUAL_ virtual
-#else
-#  define SC_VIRTUAL_ /* non-virtual */
-#endif
-
 #if defined(_MSC_VER) && !defined(SC_WIN_DLL_WARN)
 #pragma warning(push)
 #pragma warning(disable: 4251) // DLL import for std::string
@@ -164,10 +158,10 @@ public:
 
     // bind to in interface
 
-    SC_VIRTUAL_ void bind( const in_if_type& interface_ )
+    virtual void bind( const in_if_type& interface_ )
 	{ sc_port_base::bind( const_cast<in_if_type&>( interface_ ) ); }
 
-    SC_VIRTUAL_ void bind( in_if_type& interface_ )
+    virtual void bind( in_if_type& interface_ )
 	{ this->bind( const_cast<const in_if_type&>( interface_ ) ); }
 
     void operator () ( const in_if_type& interface_ )
@@ -176,7 +170,7 @@ public:
 
     // bind to parent in port
 
-    SC_VIRTUAL_ void bind( in_port_type& parent_ )
+    virtual void bind( in_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
     void operator () ( in_port_type& parent_ )
@@ -185,7 +179,7 @@ public:
 
     // bind to parent inout port
 
-    SC_VIRTUAL_ void bind( inout_port_type& parent_ )
+    virtual void bind( inout_port_type& parent_ )
 	{ sc_port_base::bind( parent_ ); }
 
     void operator () ( inout_port_type& parent_ )
@@ -225,12 +219,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-	return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
 
@@ -266,7 +256,7 @@ protected:
     //  - should only be called, when using sc_port_b explicitly
     //  - errors are detected during elaboration
 
-    SC_VIRTUAL_ void bind( base_port_type& parent_ )
+    virtual void bind( base_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
 
@@ -489,10 +479,10 @@ public:
 
     // bind to in interface
 
-    SC_VIRTUAL_ void bind( const in_if_type& interface_ )
+    virtual void bind( const in_if_type& interface_ )
 	{ sc_port_base::bind( const_cast<in_if_type&>( interface_ ) ); }
 
-    SC_VIRTUAL_ void bind( in_if_type& interface_ )
+    virtual void bind( in_if_type& interface_ )
 	{ this->bind( const_cast<const in_if_type&>( interface_ ) ); }
 
     void operator () ( const in_if_type& interface_ )
@@ -501,7 +491,7 @@ public:
 
     // bind to parent in port
 
-    SC_VIRTUAL_ void bind( in_port_type& parent_ )
+    virtual void bind( in_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
     void operator () ( in_port_type& parent_ )
@@ -510,7 +500,7 @@ public:
 
     // bind to parent inout port
 
-    SC_VIRTUAL_ void bind( inout_port_type& parent_ )
+    virtual void bind( inout_port_type& parent_ )
 	{ sc_port_base::bind( parent_ ); }
 
     void operator () ( inout_port_type& parent_ )
@@ -554,24 +544,16 @@ public:
 
     sc_event_finder& pos() const
     {
-        if ( !m_pos_finder_p )
-	{
-	    m_pos_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::posedge_event );
-	}
-	return *m_pos_finder_p;
+        return sc_event_finder::cached_create
+                ( m_pos_finder_p, *this, &in_if_type::posedge_event );
     }
 
     // use for negative edge sensitivity
 
     sc_event_finder& neg() const
     {
-        if ( !m_neg_finder_p )
-	{
-	    m_neg_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::negedge_event );
-	}
-	return *m_neg_finder_p;
+        return sc_event_finder::cached_create
+                ( m_neg_finder_p, *this, &in_if_type::negedge_event );
     }
 
 
@@ -594,12 +576,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-	return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
 
@@ -635,7 +613,7 @@ protected:
     //  - should only be called, when using sc_port_b explicitly
     //  - errors are detected during elaboration
 
-    SC_VIRTUAL_ void bind( base_port_type& parent_ )
+    virtual void bind( base_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
 private:
@@ -758,10 +736,10 @@ public:
 
     // bind to in interface
 
-    SC_VIRTUAL_ void bind( const in_if_type& interface_ )
+    virtual void bind( const in_if_type& interface_ )
 	{ sc_port_base::bind( const_cast<in_if_type&>( interface_ ) ); }
 
-    SC_VIRTUAL_ void bind( in_if_type& interface_ )
+    virtual void bind( in_if_type& interface_ )
 	{ this->bind( const_cast<const in_if_type&>( interface_ ) ); }
 
     void operator () ( const in_if_type& interface_ )
@@ -770,7 +748,7 @@ public:
 
     // bind to parent in port
 
-    SC_VIRTUAL_ void bind( in_port_type& parent_ )
+    virtual void bind( in_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
     void operator () ( in_port_type& parent_ )
@@ -779,7 +757,7 @@ public:
 
     // bind to parent inout port
 
-    SC_VIRTUAL_ void bind( inout_port_type& parent_ )
+    virtual void bind( inout_port_type& parent_ )
 	{ sc_port_base::bind( parent_ ); }
 
     void operator () ( inout_port_type& parent_ )
@@ -823,24 +801,16 @@ public:
 
     sc_event_finder& pos() const
     {
-        if ( !m_pos_finder_p )
-	{
-	    m_pos_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::posedge_event );
-	}
-	return *m_pos_finder_p;
+        return sc_event_finder::cached_create
+                ( m_pos_finder_p, *this, &in_if_type::posedge_event );
     }
 
     // use for negative edge sensitivity
 
     sc_event_finder& neg() const
     {
-        if ( !m_neg_finder_p )
-	{
-	    m_neg_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::negedge_event );
-	}
-	return *m_neg_finder_p;
+        return sc_event_finder::cached_create
+                ( m_neg_finder_p, *this, &in_if_type::negedge_event );
     }
 
 
@@ -863,12 +833,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-	return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
 
@@ -904,7 +870,7 @@ protected:
     //  - should only be called, when using sc_port_b explicitly
     //  - errors are detected during elaboration
 
-    SC_VIRTUAL_ void bind( base_port_type& parent_ )
+    virtual void bind( base_port_type& parent_ )
         { sc_port_base::bind( parent_ ); }
 
 private:
@@ -1035,22 +1001,22 @@ public:
     // write the new value
 
     void write( const data_type& value_ )
-	{ (*this)->write( value_ ); }
+      { (*this)->write( value_ ); }
 
     this_type& operator = ( const data_type& value_ )
-	{ (*this)->write( value_ ); return *this; }
+      { write( value_ ); return *this; }
 
     this_type& operator = ( const in_if_type& interface_ )
-	{ (*this)->write( interface_.read() ); return *this; }
+      { return *this = interface_.read(); }
 
     this_type& operator = ( const in_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const inout_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const this_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
 
     // set initial value (can also be called when port is not bound yet)
@@ -1072,12 +1038,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-	return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
     virtual const char* kind() const
@@ -1334,24 +1296,16 @@ public:
 
     sc_event_finder& pos() const
     {
-        if ( !m_pos_finder_p )
-	{
-	    m_pos_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::posedge_event );
-	}
-	return *m_pos_finder_p;
+        return sc_event_finder::cached_create
+                ( m_pos_finder_p, *this, &in_if_type::posedge_event );
     }
 
     // use for negative edge sensitivity
 
     sc_event_finder& neg() const
     {
-        if ( !m_neg_finder_p )
-	{
-	    m_neg_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::negedge_event );
-	}
-	return *m_neg_finder_p;
+        return sc_event_finder::cached_create
+                ( m_neg_finder_p, *this, &in_if_type::negedge_event );
     }
 
 
@@ -1373,22 +1327,22 @@ public:
     // write the new value
 
     void write( const data_type& value_ )
-	{ (*this)->write( value_ ); }
+      { (*this)->write( value_ ); }
 
     this_type& operator = ( const data_type& value_ )
-	{ (*this)->write( value_ ); return *this; }
+      { write( value_ ); return *this; }
 
     this_type& operator = ( const in_if_type& interface_ )
-	{ (*this)->write( interface_.read() ); return *this; }
+      { return *this = interface_.read(); }
 
     this_type& operator = ( const in_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const inout_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const this_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
 
     // set initial value (can also be called when port is not bound yet)
@@ -1410,12 +1364,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-	return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
     virtual const char* kind() const
@@ -1572,24 +1522,16 @@ public:
 
     sc_event_finder& pos() const
     {
-        if ( !m_pos_finder_p )
-	{
-	    m_pos_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::posedge_event );
-	}
-	return *m_pos_finder_p;
+        return sc_event_finder::cached_create
+                ( m_pos_finder_p, *this, &in_if_type::posedge_event );
     }
 
     // use for negative edge sensitivity
 
     sc_event_finder& neg() const
     {
-        if ( !m_neg_finder_p )
-	{
-	    m_neg_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::negedge_event );
-	}
-	return *m_neg_finder_p;
+        return sc_event_finder::cached_create
+                ( m_neg_finder_p, *this, &in_if_type::negedge_event );
     }
 
 
@@ -1611,22 +1553,22 @@ public:
     // write the new value
 
     void write( const data_type& value_ )
-	{ (*this)->write( value_ ); }
+      { (*this)->write( value_ ); }
 
     this_type& operator = ( const data_type& value_ )
-	{ (*this)->write( value_ ); return *this; }
+      { write( value_ ); return *this; }
 
     this_type& operator = ( const in_if_type& interface_ )
-	{ (*this)->write( interface_.read() ); return *this; }
+      { return *this = interface_.read(); }
 
     this_type& operator = ( const in_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const inout_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
     this_type& operator = ( const this_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { return *this = port_->read(); }
 
 
     // set initial value (can also be called when port is not bound yet)
@@ -1648,12 +1590,8 @@ public:
 
     sc_event_finder& value_changed() const
     {
-        if ( !m_change_finder_p )
-	{
-	    m_change_finder_p = new sc_event_finder_t<in_if_type>(
-	        *this, &in_if_type::value_changed_event );
-	}
-        return *m_change_finder_p;
+        return sc_event_finder::cached_create
+                ( m_change_finder_p, *this, &in_if_type::value_changed_event );
     }
 
     virtual const char* kind() const
@@ -1768,20 +1706,9 @@ public:
 
     // write the new value
 
-    this_type& operator = ( const data_type& value_ )
-	{ (*this)->write( value_ ); return *this; }
-
-    this_type& operator = ( const in_if_type& interface_ )
-	{ (*this)->write( interface_.read() ); return *this; }
-
-    this_type& operator = ( const in_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
-
-    this_type& operator = ( const inout_port_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
-
+    using base_type::operator=;
     this_type& operator = ( const this_type& port_ )
-	{ (*this)->write( port_->read() ); return *this; }
+      { base_type::operator=(port_); return *this; }
 
     virtual const char* kind() const
         { return "sc_out"; }
@@ -1837,8 +1764,6 @@ sc_trace( sc_trace_file* tf, const sc_inout<T>& port,
 
 } // namespace sc_core
 
-#undef SC_VIRTUAL_
-
 #if defined(_MSC_VER) && !defined(SC_WIN_DLL_WARN)
 #pragma warning(pop)
 #endif
